package cyou.chenx221.controller;

import cyou.chenx221.helper.UsernameHelper;
import cyou.chenx221.pojo.Course;
import cyou.chenx221.service.CourseService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.List;

@Controller
@RequestMapping("/course")
public class CourseController {
    private final CourseService courseService;

    @Autowired
    public CourseController(CourseService courseService) {
        this.courseService = courseService;
    }

    @GetMapping("/all")
    public String getAllCourses(Model model) {
        String username = new UsernameHelper().getCurrentUsername();
        String role = new UsernameHelper().getCurrentRole();
        if (username != null) {
            model.addAttribute("username", username);
        }
        List<Course> courses = courseService.getAllCourses();
        model.addAttribute("courses", courses);
        if (role.equals("[ROLE_admin]"))
            return "course/courseList";
        else if (role.equals("[ROLE_teacher]")) {
            return "course/courseList_t";
        } else {
            return "course/courseList_s";
        }

    }

    @GetMapping("/infomodv2")
    public String getInfoModv2(@RequestParam(value = "successMessage", required = false, defaultValue = "null") String successMessage,
                               @RequestParam(value = "errorMessage", required = false, defaultValue = "null") String errorMessage,
                               Model model) {
        String username = new UsernameHelper().getCurrentUsername();
        if (username != null) {
            model.addAttribute("username", username);
        }
        List<Course> courses = courseService.getAllCourses();
        model.addAttribute("courses", courses);
        model.addAttribute("successMessage", successMessage);
        model.addAttribute("errorMessage", errorMessage);
        String role = new UsernameHelper().getCurrentRole();
        if (role.equals("[ROLE_admin]"))
            return "course/courseopv2";
        else if (role.equals("[ROLE_teacher]")) {
            return "course/courseopv2_t";
        } else {
            return "course/courseopv2_s";
        }
    }

    @PostMapping(value = "/delinfo", produces = "application/x-www-form-urlencoded;charset=UTF-8")
    public String DeleteCourseInfo(@RequestParam(value = "id") int id,
                                   Model model) {
        Course temp = courseService.getCourseById(id);
        if (id <= 0 || temp == null) {
            model.addAttribute("errorMessage", "学生信息删除失败,可能是ID非法或者学生已删除");
        } else {
            int status_code = courseService.deleteCourse(id);
            if (status_code == 1) {
                model.addAttribute("successMessage", "学生信息删除成功");
            } else if (status_code == 0) {
                model.addAttribute("errorMessage", "学生信息删除失败,可能是ID非法或者学生已删除");
            } else {
                model.addAttribute("errorMessage", "学生信息删除异常");
                System.out.println("DEBUG:异常删除，多条记录被删除");
            }
        }
        return "redirect:/course/infomodv2";
    }


    @PostMapping(value = "/addinfo", produces = "application/x-www-form-urlencoded;charset=UTF-8")
    public String InsertCourseInfo(@RequestParam(value = "name", defaultValue = "null", required = false) String name,
                                   @RequestParam(value = "description", defaultValue = "null", required = false) String description,
                                   Model model) {
        Course course;
        course = new Course(name, description);
        courseService.insertCourse(course);
        model.addAttribute("successMessage", "学生信息添加成功");
        return "redirect:/course/infomodv2";
    }


    @PostMapping(value = "/modinfo", produces = "application/x-www-form-urlencoded;charset=UTF-8")
    public String UpdateModInfo(@RequestParam(value = "name", defaultValue = "null", required = false) String name,
                                @RequestParam(value = "description", defaultValue = "null", required = false) String description,
                                @RequestParam(value = "id", defaultValue = "-1", required = false) int id,
                                Model model) {

        Course course;
        if (name.equals("null")) name = null;
        if (description.equals("null")) description = null;
        if (id <= -1) {
            model.addAttribute("errorMessage", "ID非法，课程信息修改失败");
        } else {
            course = new Course(id, name, description);
            int status_code = courseService.updateCourse(course);
            if (status_code == 1) {
                model.addAttribute("successMessage", "课程信息修改成功");
            } else if (status_code == 0) {
                model.addAttribute("errorMessage", "课程信息修改失败，可能是ID非法或者课程不存在");
            } else {
                model.addAttribute("errorMessage", "课程信息修改异常");
                System.out.println("DEBUG:课程信息修改异常，存在多条记录被修改的情况");
            }
        }
        return "redirect:/course/infomodv2";
    }

    @PostMapping(value = "/query", produces = "application/x-www-form-urlencoded;charset=UTF-8")
    public String getQueryCourses(@RequestParam(value = "name", defaultValue = "null", required = false) String name,
                                  @RequestParam(value = "description", defaultValue = "null", required = false) String description,
                                  @RequestParam(value = "id", defaultValue = "-1", required = false) int id,
                                  Model model) {
        String username = new UsernameHelper().getCurrentUsername();
        if (username != null) {
            model.addAttribute("username", username);
        }
        if (name.equals("null")) name = null;
        if (description.equals("null")) description = null;
        if (name == null && description == null && id <= 0) {
            model.addAttribute("errorMessage", "查询条件不可全为空");
            return "redirect:/course/infomodv2";
        }
        List<Course> courses;
        Course course;
        course = new Course((id == -1 ? null : id), name, description);
        courses = courseService.getQueryCourses(course);
        model.addAttribute("courses", courses);
        String role = new UsernameHelper().getCurrentRole();
        if (role.equals("[ROLE_admin]"))
            return "course/courseQueryResult";
        else if (role.equals("[ROLE_teacher]")) {
            return "course/courseQueryResult_t";
        } else {
            return "course/courseQueryResult_s";
        }
    }
}
