package cyou.chenx221.controller;

import cyou.chenx221.helper.UsernameHelper;
import cyou.chenx221.mapper.UserMapper;
import cyou.chenx221.pojo.User;
import cyou.chenx221.service.PersonService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.io.InputStreamResource;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.HashMap;
import java.util.Map;

@Controller
@RequestMapping("/person")
public class PersonController {
    private final PersonService personService;
    @Autowired
    private UserMapper userMapper;

    @Autowired
    public PersonController(PersonService personService) {
        this.personService = personService;
    }

    @GetMapping("/settings")
    public String getPersonSettings(Model model) {
        String username = new UsernameHelper().getCurrentUsername();
        if (username != null) {
            model.addAttribute("username", username);
        }
        String role = new UsernameHelper().getCurrentRole();
        if (role.equals("[ROLE_admin]"))
            return "person/person";
        else if (role.equals("[ROLE_teacher]")) {
            return "person/person_t";
        } else {
            return "person/person_s";
        }
    }

    @ResponseBody
    @PostMapping("/uploadAvatar")
    public ResponseEntity<Map<String, String>> uploadAvatar(@RequestParam("avatar") MultipartFile file) {
        Map<String, String> response = new HashMap<>();
        //TODO: 清理没有关联的图像资源
        if (!file.isEmpty()) {
            try {
                String object = personService.uploadAvatar(file);
                User user = new User();
                user.setUsername(new UsernameHelper().getCurrentUsername());
                user.setAvatar(object);
                userMapper.setAvatar(user);
                return ResponseEntity.ok(response);
            } catch (Exception e) {
                String errorMessage = "Error uploading file: " + e.getMessage();
                response.put("message", errorMessage);
                return ResponseEntity.status(HttpStatus.INTERNAL_SERVER_ERROR).body(response);
            }
        } else {
            response.put("message", "No file received");
            return ResponseEntity.badRequest().body(response);
        }
    }

    @ResponseBody
    @GetMapping("/image")
    public ResponseEntity<InputStreamResource> getPersonImage() {
        String objectName = userMapper.getAvatar(new UsernameHelper().getCurrentUsername());
        try {
            InputStreamResource imageResource = personService.getImage(objectName);
            return ResponseEntity.ok().body(imageResource);
        } catch (Exception e) {
            return ResponseEntity.notFound().build();
        }
    }

}
