package cyou.chenx221.controller;

import cyou.chenx221.helper.CsvHelper;
import cyou.chenx221.helper.UsernameHelper;
import cyou.chenx221.pojo.Course;
import cyou.chenx221.pojo.Score;
import cyou.chenx221.pojo.Student;
import cyou.chenx221.service.ScoreService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.io.IOException;
import java.util.List;

@Controller
@RequestMapping("/score")
public class ScoreController {
    private final ScoreService scoreService;

    @Autowired
    public ScoreController(ScoreService scoreService) {
        this.scoreService = scoreService;
    }

    @GetMapping("/all")
    public String getAllScore(Model model) {
        String username = new UsernameHelper().getCurrentUsername();
        if (username != null) {
            model.addAttribute("username", username);
        }
        String role = new UsernameHelper().getCurrentRole();
        if (role.equals("[ROLE_admin]")) {
            model.addAttribute("scores", scoreService.getAllScores());
            return "score/scoreList";
        } else if (role.equals("[ROLE_teacher]")) {
            model.addAttribute("scores", scoreService.getAllScores());
            return "score/scoreList_t";
        } else {
            model.addAttribute("scores", scoreService.getQueryScore(scoreService.getStudentIdByUsername(username)));
            return "score/scoreList_s";
        }
    }

    @GetMapping("/infomodv2")
    public String getInfoModv2(@RequestParam(value = "successMessage", required = false, defaultValue = "null") String successMessage,
                               @RequestParam(value = "errorMessage", required = false, defaultValue = "null") String errorMessage,
                               Model model) {
        String username = new UsernameHelper().getCurrentUsername();
        if (username != null) {
            model.addAttribute("username", username);
        }
        model.addAttribute("scores", scoreService.getAllScores());
        model.addAttribute("successMessage", successMessage);
        model.addAttribute("errorMessage", errorMessage);
        String role = new UsernameHelper().getCurrentRole();
        if (role.equals("[ROLE_admin]"))
            return "score/scoreopv2";
        else {
            return "score/scoreopv2_t";
        }
    }

    @PostMapping(value = "/query", produces = "application/x-www-form-urlencoded;charset=UTF-8")
    public String getQueryScores(@RequestParam(value = "name", defaultValue = "null", required = false) String name,//学生姓名
                                 @RequestParam(value = "coursename", defaultValue = "null", required = false) String course_name,//课程名
                                 @RequestParam(value = "scoreid", defaultValue = "-1", required = false) int score_id,//成绩ID
                                 @RequestParam(value = "studentid", defaultValue = "-1", required = false) int student_id,//学号
                                 @RequestParam(value = "courseid", defaultValue = "-1", required = false) int course_id,//课程ID
                                 @RequestParam(value = "min", defaultValue = "-1", required = false) int min,//最低分
                                 @RequestParam(value = "max", defaultValue = "-1", required = false) int max,//最高分
                                 @RequestParam(value = "range_enabler", defaultValue = "false", required = false) boolean range_enabler,//是否启用分数范围
                                 Model model) throws IOException {

        //username helper start
        String username = new UsernameHelper().getCurrentUsername();
        if (username != null) {
            model.addAttribute("username", username);
        }
        //username helper end

        if (name.equals("null")) {
            name = null;
        }
        if (course_name.equals("null")) {
            course_name = null;
        }
//        System.out.println("name: " + name + " course_name: " + course_name + " score_id: " + score_id + " student_id: " + student_id + " course_id: " + course_id + " min: " + min + " max: " + max + " range_enabler: " + range_enabler);
        if (score_id == -1 && student_id == -1 && course_id == -1 && !range_enabler && course_name == null && name == null) {
            model.addAttribute("errorMessage", "查询条件不可全为空");
            return "redirect:/score/infomodv2";
        }

        List<Score> scores;
        Score score;

        score = new Score((score_id == -1 ? null : score_id), ((student_id == -1 && name == null) ? null : (new Student((student_id == -1 ? null : student_id), name))), (course_id == -1 && course_name == null) ? null : (new Course((course_id == -1 ? null : course_id), course_name)), range_enabler ? (min == -1 ? null : min) : null, range_enabler ? (max == -1 ? null : max) : null);

        scores = scoreService.getQueryScores(score);
        String download = new CsvHelper().generateScoreCSVFile(scores);
        model.addAttribute("download", download);
        model.addAttribute("scores", scores);
        String role = new UsernameHelper().getCurrentRole();
        if (role.equals("[ROLE_admin]"))
            return "score/scoreQueryResult";
        else {
            return "score/scoreQueryResult_t";
        }
    }

    @PostMapping(value = "/modinfo", produces = "application/x-www-form-urlencoded;charset=UTF-8")
    public String UpdateScoreInfo(@RequestParam(value = "score", defaultValue = "-1", required = false) int score,
                                  @RequestParam(value = "id", defaultValue = "-1", required = false) int id,//成绩ID
                                  Model model) {
        if (id <= 0) {
            model.addAttribute("errorMessage", "修改失败，ID非法");
        } else if (score > 100 || score < 0) {
            model.addAttribute("errorMessage", "修改失败，分数非法");
        } else {
            Score score1 = new Score(id, score);
            int status_code = scoreService.updateScore(score1);
            if (status_code == 1) {
                model.addAttribute("successMessage", "修改成功");
            } else {
                model.addAttribute("errorMessage", "修改失败，可能是成绩不存在");
            }
        }
        return "redirect:/score/infomodv2";
    }

    @PostMapping(value = "/delinfo", produces = "application/x-www-form-urlencoded;charset=UTF-8")
    public String DeleteScoreInfo(@RequestParam(value = "id") int id, //成绩ID
                                  Model model) {
        Score temp = scoreService.getScoreById(id);
        if (id <= 0 || temp == null) {
            model.addAttribute("errorMessage", "成绩信息删除失败,可能是ID非法或者成绩已删除");
        } else {
            int status_code = scoreService.deleteScore(id);
            if (status_code == 1) {
                model.addAttribute("successMessage", "学生信息删除成功");
            } else {
                model.addAttribute("errorMessage", "学生信息删除异常");
            }
        }
        return "redirect:/score/infomodv2";
    }

    @PostMapping(value = "/addinfo", produces = "application/x-www-form-urlencoded;charset=UTF-8")
    public String InsertCourseInfo(@RequestParam(value = "studentid", defaultValue = "-1", required = false) int studentid,
                                   @RequestParam(value = "courseid", defaultValue = "-1", required = false) int courseid,
                                   @RequestParam(value = "score", defaultValue = "-1", required = false) int score,
                                   Model model) {
        if (studentid <= 0) {
            model.addAttribute("errorMessage", "添加失败，学号非法");
        } else if (courseid <= 0) {
            model.addAttribute("errorMessage", "添加失败，课程ID非法");
        } else if (score > 100 || score < 0) {
            model.addAttribute("errorMessage", "添加失败，分数非法");
        } else {
            Score score1 = new Score(new Student(studentid), new Course(courseid), score);
            int status_code = scoreService.insertScore(score1);
            if (status_code == 1) {
                model.addAttribute("successMessage", "学生信息添加成功");
            } else {
                model.addAttribute("errorMessage", "学生信息添加失败");
            }
        }
        String role = new UsernameHelper().getCurrentRole();
        if (role.equals("[ROLE_admin]"))
            return "redirect:/score/infomodv2";
        else {
            return "redirect:/score/infomodv2_t";
        }
    }
}
