package cyou.chenx221.controller;

import cyou.chenx221.helper.UsernameHelper;
import cyou.chenx221.mapper.UserMapper;
import cyou.chenx221.pojo.Student;
import cyou.chenx221.service.StudentService;
import cyou.chenx221.service.TeacherService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import java.sql.Date;
import java.util.List;

@Controller
@RequestMapping("/student")
public class StudentController {
    private final StudentService studentService;
    private final TeacherService teacherService;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    public StudentController(StudentService studentService, TeacherService teacherService) {
        this.studentService = studentService;
        this.teacherService = teacherService;
    }

    @GetMapping("/all")//学生信息查询（不带条件）(排除removed的学生）
    public String getAllStudents(Model model) {
        String username = new UsernameHelper().getCurrentUsername();
        if (username != null) {
            model.addAttribute("username", username);
        }
        String role = new UsernameHelper().getCurrentRole();
        if (role.equals("[ROLE_admin]")) {
            List<Student> students = studentService.getAllStudents();
            model.addAttribute("students", students);
            return "student/studentList";
        } else if (role.equals("[ROLE_teacher]")) {
            String classes = teacherService.getClassesById(userMapper.getDetailByUsername(username));
            List<Student> students = studentService.getAllStudentsT(classes);
            model.addAttribute("classes", classes);
            model.addAttribute("students", students);
            return "student/studentList_t";
        } else {
            int id = studentService.getIdByUsername(username);
            Student student = studentService.getStudentById(id);
            model.addAttribute("student", student);
            return "student/studentList_s";
        }
    }

    @GetMapping("/queryinfo")//处理访问学生信息查询
    public String getQueryInfo(Model model) {
        String username = new UsernameHelper().getCurrentUsername();
        if (username != null) {
            model.addAttribute("username", username);
        }
        String role = new UsernameHelper().getCurrentRole();
        if (role.equals("[ROLE_admin]"))
            return "student/studentListQuery";
        else {
            model.addAttribute("classes", teacherService.getClassesById(userMapper.getDetailByUsername(username)));
            return "student/studentListQuery_t";
        }
    }

    @PostMapping(value = "/query", produces = "application/x-www-form-urlencoded;charset=UTF-8")//学生信息查询（带条件）
    public String getQueryStudents(@RequestParam(value = "name", defaultValue = "null", required = false) String name,
                                   @RequestParam(value = "birthday", defaultValue = "null", required = false) String birthdayStr,
                                   @RequestParam(value = "inlineRadioOptions", defaultValue = "null", required = false) String option,
                                   @RequestParam(value = "student-id", defaultValue = "-1", required = false) int id,
                                   @RequestParam(value = "phone", defaultValue = "null", required = false) String phone,
                                   @RequestParam(value = "classes", defaultValue = "", required = false) List<String> classes,
                                   Model model) {
        String username = new UsernameHelper().getCurrentUsername();
        if (username != null) {
            model.addAttribute("username", username);
        }
        Date birthday = null;
        if (!"null".equals(birthdayStr)) {
            birthday = Date.valueOf(birthdayStr);
        }
        List<Student> students;
        Student student;
        if (classes.size() == 0) {
            student = new Student(id, name, option, birthday, phone);
            System.out.println("id:" + student.getId() + " name:" + student.getName() + " sex:" + student.getSex() + " birthday:" + student.getBirthday() + " phone:" + student.getPhone());
            if (student.getId() == null) {
                System.out.println("id is null");
            }
            if (student.getName() == null) {
                System.out.println("name is null");
            }
            if (student.getSex() == null) {
                System.out.println("sex is null");
            }
            if (student.getBirthday() == null) {
                System.out.println("birthday is null");
            }
            if (student.getPhone() == null) {
                System.out.println("phone is null");
            }

            students = studentService.getQueryStudents(student);
        } else {
            if (classes.size() == 1 && classes.get(0).equals("")) classes = null;
            student = new Student(classes);
            students = studentService.getQueryStudents2(student);
        }
        model.addAttribute("students", students);
        String role = new UsernameHelper().getCurrentRole();
        if (role.equals("[ROLE_admin]"))
            return "student/studentQueryResult";
        else {
            return "student/studentQueryResult_t";
        }
    }

    @GetMapping("/modstep1")//处理学生信息修改页
    public String getModStep1(@RequestParam(value = "successMessage", required = false, defaultValue = "null") String successMessage,
                              @RequestParam(value = "errorMessage", required = false, defaultValue = "null") String errorMessage,
                              Model model) {
        String username = new UsernameHelper().getCurrentUsername();
        if (username != null) {
            model.addAttribute("username", username);
        }
        List<Student> students = studentService.getAllStudents();//和前面显示全部学生一样，页面需要学生信息的显示
        model.addAttribute("students", students);
        // 将消息添加到Model中
        model.addAttribute("successMessage", successMessage);
        model.addAttribute("errorMessage", errorMessage);
        String role = new UsernameHelper().getCurrentRole();
        if (role.equals("[ROLE_admin]"))
            return "student/studentMod";
        else {
            model.addAttribute("classes", teacherService.getClassesById(userMapper.getDetailByUsername(username)));
            return "student/studentMod_t";
        }
    }

    @PostMapping(value = "/modinfo", produces = "application/x-www-form-urlencoded;charset=UTF-8")//处理学生信息修改请求
    public String UpdateModInfo(@RequestParam(value = "name", defaultValue = "null", required = false) String name,
                                @RequestParam(value = "birthday", defaultValue = "null", required = false) String birthdayStr,
                                @RequestParam(value = "sex", defaultValue = "null", required = false) String sex,
                                @RequestParam(value = "id", defaultValue = "-1", required = false) int id,
                                @RequestParam(value = "phone", defaultValue = "null", required = false) String phone,
                                @RequestParam(value = "classes", defaultValue = "", required = false) String classes,
                                Model model) {
        Date birthday = null;
        if (!"null".equals(birthdayStr)) {
            birthday = Date.valueOf(birthdayStr);
        }
        System.out.println(birthday);//debug
        Student student;
        if (name.equals("null")) name = null;
        if (sex.equals("null")) sex = null;
        if (phone.equals("null")) phone = null;
        if (classes.equals("")) classes = null;

        if (id <= -1) {
            model.addAttribute("errorMessage", "学生信息修改失败");
        } else {
            student = new Student(id, name, sex, birthday, phone, classes);
            studentService.updateStudent(student);
            model.addAttribute("successMessage", "学生信息修改成功");
        }
        String role = new UsernameHelper().getCurrentRole();
        if (role.equals("[ROLE_admin]"))
            return "redirect:/student/infomodv2";
        else {
            return "redirect:/student/infomodv2_t";
        }
    }

    @GetMapping("/infomodv2")//处理学生信息修改页
    public String getInfoModv2(@RequestParam(value = "successMessage", required = false, defaultValue = "null") String successMessage,
                               @RequestParam(value = "errorMessage", required = false, defaultValue = "null") String errorMessage,
                               Model model) {
        String username = new UsernameHelper().getCurrentUsername();
        if (username != null) {
            model.addAttribute("username", username);
        }

        model.addAttribute("successMessage", successMessage);
        model.addAttribute("errorMessage", errorMessage);
        String role = new UsernameHelper().getCurrentRole();
        if (role.equals("[ROLE_admin]")) {
            List<Student> students = studentService.getAllStudents();
            model.addAttribute("students", students);
            return "student/studentinfomodv2";
        } else {
            String classes = teacherService.getClassesById(userMapper.getDetailByUsername(username));
            List<Student> students = studentService.getAllStudentsT(classes);
            model.addAttribute("classes", classes);
            model.addAttribute("students", students);
            return "student/studentinfomodv2_t";
        }
    }

    @PostMapping(value = "/delinfo", produces = "application/x-www-form-urlencoded;charset=UTF-8")
    public String UpdateModInfo(@RequestParam(value = "id") int id,
                                @RequestParam(value = "cla", required = false, defaultValue = "null") String classes,
                                Model model) {
        Student temp = studentService.getStudentById(id);
        if (id <= 0 || temp == null) {
            model.addAttribute("errorMessage", "学生信息删除失败,可能是ID非法或者学生已删除");
        } else {
            if (classes.equals("null")) {
                studentService.deleteStudent(id);
                model.addAttribute("successMessage", "学生信息删除成功");
            } else {
                if (temp.getClasses().equals(classes)) {
                    studentService.deleteStudent(id);
                    model.addAttribute("successMessage", "学生信息删除成功");
                } else {
                    model.addAttribute("errorMessage", "学生信息删除失败,这名学生不属于您的班级");
                }
            }
        }
        String role = new UsernameHelper().getCurrentRole();
        if (role.equals("[ROLE_admin]"))
            return "redirect:/student/infomodv2";
        else {
            return "redirect:/student/infomodv2_t";
        }
    }

    @PostMapping(value = "/addinfo", produces = "application/x-www-form-urlencoded;charset=UTF-8")
    public String InsertStudentInfo(@RequestParam(value = "name") String name,
                                    @RequestParam(value = "birthday") Date birthday,
                                    @RequestParam(value = "sex") String sex,
                                    @RequestParam(value = "phone") String phone,
                                    @RequestParam(value = "classes") String classes,
                                    Model model) {
        Student student;
        String classesPattern = "[1-5]班";// 使用正则表达式验证班级格式

        //格式合法性检查
        if (!sex.equals("男") && !sex.equals("女")) {
            model.addAttribute("errorMessage", "性别格式不正确，仅允许'男'或'女'");
        } else if (!classes.matches(classesPattern)) {
            model.addAttribute("errorMessage", "班级格式不正确，格式应为?班（?为1~5之间的数字）");
        } else {
            student = new Student(name, sex, birthday, phone, classes);
            studentService.insertStudent(student);
            model.addAttribute("successMessage", "学生信息添加成功");
        }
        String role = new UsernameHelper().getCurrentRole();
        if (role.equals("[ROLE_admin]"))
            return "redirect:/student/infomodv2";
        else {
            return "redirect:/student/infomodv2_t";
        }
    }
}
