package cyou.chenx221.controller;

import cyou.chenx221.helper.UsernameHelper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import java.io.File;
import java.text.DecimalFormat;

@Controller
@RequestMapping("/system")
public class SystemController {

    @GetMapping("/settings")
    public String getSettings(Model model, @ModelAttribute("message") String message) {
        String username = new UsernameHelper().getCurrentUsername();
        if (username != null) {
            model.addAttribute("username", username);
        }
        if(message!=null && !message.isEmpty()){
            model.addAttribute("message", message);
        }
        model.addAttribute("trashSize", formatFileSize(getTrashSize()));
        return "system/settings";
    }

    public long getTrashSize() {
        File directory = new File("file");
        long size = 0;
        if (directory.exists() && directory.isDirectory()) {
            File[] files = directory.listFiles();
            if (files != null) {
                for (File file : files) {
                    if (file.isFile()) {
                        size += file.length();
                    }
                }
            }
        }
        return size;
    }

    public String formatFileSize(long size) {
        DecimalFormat decimalFormat = new DecimalFormat("#.##");
        double kiloByte = size / 1024.0;
        double megaByte = kiloByte / 1024.0;
        double gigaByte = megaByte / 1024.0;
        double teraByte = gigaByte / 1024.0;

        if (teraByte > 1) {
            return decimalFormat.format(teraByte) + " TB";
        } else if (gigaByte > 1) {
            return decimalFormat.format(gigaByte) + " GB";
        } else if (megaByte > 1) {
            return decimalFormat.format(megaByte) + " MB";
        } else if (kiloByte > 1) {
            return decimalFormat.format(kiloByte) + " KB";
        } else {
            return size + " B";
        }
    }

    @GetMapping("/clean")
    public String getClean(RedirectAttributes redirectAttributes) {
        //todo: clean work path file/*.csv
        File directory = new File("file");

        // 检查目录是否存在
        if (directory.exists() && directory.isDirectory()) {
            // 获取目录中的所有文件
            File[] files = directory.listFiles();

            // 遍历文件并删除以 ".csv" 结尾的文件
            if (files != null) {
                for (File file : files) {
                    if (file.isFile()) {
                        if (file.delete()) {
                            System.out.println("REPORT: Trash " + file.getName() + " deleted");
                        } else {
                            System.out.println("REPORT: Trash " + file.getName() + " delete failed");
                        }
                    }
                }
            } else {
                System.out.println("REPORT: no trash file found");
            }
        }
        redirectAttributes.addFlashAttribute("message", "清理完毕");
        return "redirect:/system/settings";
    }

}
