package cyou.chenx221.controller;

import cyou.chenx221.helper.CsvHelper;
import cyou.chenx221.helper.UsernameHelper;
import cyou.chenx221.pojo.Course;
import cyou.chenx221.pojo.Teacher;
import cyou.chenx221.service.TeacherService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.sql.Date;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.List;

@Controller
@RequestMapping("/teacher")
public class TeacherController {
    private final TeacherService teacherService;

    @Autowired
    public TeacherController(TeacherService teacherService) {
        this.teacherService = teacherService;
    }

    @GetMapping("/manage")
    public String manageTeacherInfo(@RequestParam(value = "successMessage", required = false, defaultValue = "null") String successMessage,
                                    @RequestParam(value = "errorMessage", required = false, defaultValue = "null") String errorMessage,
                                    Model model) {
        String username = new UsernameHelper().getCurrentUsername();
        if (username != null) {
            model.addAttribute("username", username);
        }
        model.addAttribute("successMessage", successMessage);
        model.addAttribute("errorMessage", errorMessage);
        List<Teacher> teacherList = teacherService.getAllTeachers();
        model.addAttribute("teacherList", teacherList);
        return "teacher/teachermanage";
    }

    @PostMapping(value = "/query", produces = "application/x-www-form-urlencoded;charset=UTF-8")
    public String postQuery(@RequestParam(value = "id", defaultValue = "-1", required = false) int id,//教师id
                            @RequestParam(value = "name", defaultValue = "null", required = false) String name,//教师姓名
                            @RequestParam(value = "sex", defaultValue = "null", required = false) String sex,//性别
                            @RequestParam(value = "birthday_begin", defaultValue = "null", required = false) String birthday_begin_str,//日期起始
                            @RequestParam(value = "birthday_end", defaultValue = "null", required = false) String birthday_end_str,//日期结束
                            @RequestParam(value = "course_id", defaultValue = "-1", required = false) int course_id,//课程ID
                            @RequestParam(value = "course_name", defaultValue = "null", required = false) String course_name,//课程名
                            @RequestParam(value = "classes", defaultValue = "null", required = false) String classes,//授课班级
                            Model model) throws IOException {

        //username helper start
        String username = new UsernameHelper().getCurrentUsername();
        if (username != null) {
            model.addAttribute("username", username);
        }
        //username helper end

        String pattern = "yyyy-MM-dd"; // 日期字符串的格式，与HTML的input type为date的格式相对应
        DateFormat dateFormat = new SimpleDateFormat(pattern);
        Date birthdayBegin = null, birthdayEnd = null;
        try {
            if (!birthday_begin_str.equals("null")) {
                java.util.Date parsedBeginDate = dateFormat.parse(birthday_begin_str);
                birthdayBegin = new Date(parsedBeginDate.getTime());
            }
            if (!birthday_end_str.equals("null")) {
                java.util.Date parsedEndDate = dateFormat.parse(birthday_end_str);
                birthdayEnd = new Date(parsedEndDate.getTime());
            }
        } catch (ParseException e) {
            e.printStackTrace();
        }
        if (name.equals("null")) {
            name = null;
        }
        if (sex.equals("null")) {
            sex = null;
        }
        if (course_name.equals("null")) {
            course_name = null;
        }
        if (classes.equals("null")) {
            classes = null;
        }

        if (id <= 0 && name == null && sex == null && birthdayBegin == null && birthdayEnd == null && course_id <= 0 && course_name == null && classes == null) {
            model.addAttribute("errorMessage", "ID非法或查询条件全为空");
            return "redirect:/teacher/manage";
        } else if (birthdayBegin != null && birthdayEnd != null) {
            if (birthdayBegin.compareTo(birthdayEnd) > 0) {//birthdayBegin > birthdayEnd
                model.addAttribute("errorMessage", "日期范围异常");
                return "redirect:/teacher/manage";
            }
        }
        Teacher teacher = new Teacher(id, name, sex, (course_id == -1 && course_name == null ? null : (new Course(course_id, course_name))), classes, birthdayBegin, birthdayEnd);
        List<Teacher> teachers = teacherService.getQueryTeachers(teacher);
        model.addAttribute("teachers", teachers);
        model.addAttribute("download", new CsvHelper().generateTeacherCSVFile(teachers));
        return "teacher/teacherQueryResult";
    }

    @ResponseBody
    @PostMapping(value = "/mod", produces = "text/plain;charset=UTF-8")
    public ResponseEntity<String> postEdit(@RequestParam(value = "id") int id,
                                           @RequestParam(value = "name", defaultValue = "null", required = false) String name,
                                           @RequestParam(value = "birthday", defaultValue = "null", required = false) String birthday_str,
                                           @RequestParam(value = "course_id", defaultValue = "-1", required = false) int course_id,
                                           @RequestParam(value = "classes", defaultValue = "null", required = false) String classes) throws ParseException {
        // 处理表单字段数据
        String classesPattern = "[1-5]班";// 使用正则表达式验证班级格式

        if (id <= 0) {
            return ResponseEntity.badRequest().body("ID非法");
        } else if (name.equals("null") && birthday_str.equals("null") && course_id == -1 && classes.equals("null")) {
            return ResponseEntity.badRequest().body("修改内容为空");
        }
        Date birthday = null;
        if (!birthday_str.equals("null")) {
            birthday = new Date(new SimpleDateFormat("yyyy-MM-dd").parse(birthday_str).getTime());
        }
        if (!classes.equals("null")) {
            if (!classes.matches(classesPattern)) {
                return ResponseEntity.badRequest().body("班级格式非法");
            }
        }
        if (name.equals("null")) {
            name = null;
        }
        if (classes.equals("null")) {
            classes = null;
        }

        //注
        //这里不考虑一个班一个科目不允许多名教师，少了个班级表+数据表外键搞得不好，所以这里偷懒了...

        Teacher teacher = new Teacher(id, name, birthday, new Course(course_id), classes);
        int status_code = teacherService.updateTeacher(teacher);
        if (status_code == 0) {
            return ResponseEntity.badRequest().body("修改失败");
        } else {
            return ResponseEntity.ok("修改成功");
        }
    }

    @ResponseBody
    @PostMapping(value = "/del", produces = "text/plain;charset=UTF-8")
    public ResponseEntity<String> postDelete(@RequestParam(value = "id") int id) {
        if (id <= 0) {
            return ResponseEntity.badRequest().body("ID非法");
        }
        int status_code = teacherService.deleteTeacher(id);
        if (status_code == 0) {
            return ResponseEntity.badRequest().body("删除失败");
        } else {
            return ResponseEntity.ok("删除成功");
        }
    }

    @ResponseBody
    @PostMapping(value = "/add", produces = "text/plain;charset=UTF-8")
    public ResponseEntity<String> postAdd(@RequestParam(value = "name") String name,
                                          @RequestParam(value = "sex") String sex,
                                          @RequestParam(value = "birthday") String birthday_str,
                                          @RequestParam(value = "course_id") int course_id,
                                          @RequestParam(value = "classes") String classes) throws ParseException {
        // 处理表单字段数据
        String classesPattern = "[1-5]班";// 使用正则表达式验证班级格式
        if (!classes.matches(classesPattern)) {
            return ResponseEntity.badRequest().body("班级格式非法");
        }
        if (course_id <= 0) {
            return ResponseEntity.badRequest().body("科目ID非法");
        }
        Date birthday = new Date(new SimpleDateFormat("yyyy-MM-dd").parse(birthday_str).getTime());

        Teacher teacher = new Teacher(name, sex, birthday, new Course(course_id), classes);
        int status_code = teacherService.addTeacher(teacher);
        if (status_code == 0) {
            return ResponseEntity.badRequest().body("添加失败");
        } else {
            return ResponseEntity.ok("添加成功");
        }
    }
}
