package cyou.chenx221.controller;

import cyou.chenx221.helper.UsernameHelper;
import cyou.chenx221.mapper.UserMapper;
import cyou.chenx221.pojo.Password;
import cyou.chenx221.pojo.User;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;

import java.security.NoSuchAlgorithmException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Controller
public class UserController {
    @Autowired
    private UserMapper userMapper;

    @RequestMapping(path = "/login", method = {RequestMethod.GET}) //登录
    public String login() {
        return "login";
    }


    @RequestMapping(path = "/signup", method = {RequestMethod.POST}) //注册
    public String register(@RequestParam("username") String username, @RequestParam("password") String password, @RequestParam("re-password") String repassword, Model model) throws NoSuchAlgorithmException {
        model.addAttribute("comeFrom", "reg");
        //检查两次输入的密码是否相同
        if (!password.equals(repassword)) {
            model.addAttribute("errorMessage", "两次输入的密码不一致，请重新输入。");
            return "login";
        }
        //检查用户名是否已经存在
        User user = userMapper.getUserByUsername(username);
        if (user == null) {
            //检查密码强度(8~32位)，需要有数字和字母，允许大小写英文字母、数字、密码常见符号
            if (password.length() < 8 || password.length() > 32) {
                model.addAttribute("errorMessage", "密码长度不符合要求，请重新输入。");
                return "login";
            } else {
                //检查密码是否符合要求
                boolean hasNumber = false;
                boolean hasLetter = false;
                boolean hasinValidChar = false;
                for (int i = 0; i < password.length(); i++) {
                    char c = password.charAt(i);
                    if (c >= '0' && c <= '9') {
                        hasNumber = true;
                    } else if ((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z')) {
                        hasLetter = true;
                    } else if (c == '!' || c == '@' || c == '#' || c == '$' || c == '%' || c == '^' || c == '&' || c == '*' || c == '.') {
                        int a = 1;//do nothing
                    } else {
                        hasinValidChar = true;
                    }
                }
                if (!hasNumber || !hasLetter || hasinValidChar) {
                    model.addAttribute("errorMessage", "密码强度不符合要求，请重新输入。要求：8~32位，需要有数字和字母，允许大小写英文字母、数字、密码常见符号");
                    return "login";
                }
                // start to register
                Password pw = new Password(password);
                System.out.println("username:" + username + " password:" + pw.getEncryptedPassword()); //debug
                user = new User(username, pw.getEncryptedPassword());
                userMapper.insertUser(user);
                model.addAttribute("comeFrom", "log");
                model.addAttribute("successMessage", "注册成功");
                return "login";
            }
        } else {
            model.addAttribute("errorMessage", "用户已存在，请重新输入。");
            return "login";
        }
    }

    @RequestMapping(path = "/user/usermanage", method = {RequestMethod.GET})
    public String userManage(@RequestParam(value = "successMessage", required = false, defaultValue = "null") String successMessage,
                             @RequestParam(value = "errorMessage", required = false, defaultValue = "null") String errorMessage,
                             Model model) {
        String username = new UsernameHelper().getCurrentUsername();
        if (username != null) {
            model.addAttribute("username", username);
        }
        List<User> userList = userMapper.getAllUsers();
        model.addAttribute("userList", userList);
        model.addAttribute("successMessage", successMessage);
        model.addAttribute("errorMessage", errorMessage);
        return "user/usermanage";
    }

    @RequestMapping(path = "/user/create", method = {RequestMethod.POST})
    public String createUser(@RequestParam("username") String username,
                             @RequestParam("password") String password,
                             @RequestParam("re_password") String repassword,
                             @RequestParam(value = "roleid", defaultValue = "-1", required = false) int roleid,
                             @RequestParam("role") String role,
                             Model model) throws NoSuchAlgorithmException {
        //check
        if (!password.equals(repassword)) {
            model.addAttribute("errorMessage", "两次输入的密码不一致，请重新输入。");
            return "redirect:/user/usermanage";
        }

        User user = userMapper.getUserByUsername(username); //check if user exists
        if (user == null) {
            //检查密码强度(8~32位)，需要有数字和字母，允许大小写英文字母、数字、密码常见符号
            if (password.length() < 8 || password.length() > 32) {
                model.addAttribute("errorMessage", "密码长度不符合要求，请重新输入。");
                return "redirect:/user/usermanage";
            } else {
                //检查密码是否符合要求
                boolean hasNumber = false;
                boolean hasLetter = false;
                boolean hasinValidChar = false;
                for (int i = 0; i < password.length(); i++) {
                    char c = password.charAt(i);
                    if (c >= '0' && c <= '9') {
                        hasNumber = true;
                    } else if ((c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z')) {
                        hasLetter = true;
                    } else if (c == '!' || c == '@' || c == '#' || c == '$' || c == '%' || c == '^' || c == '&' || c == '*' || c == '.') {
                        int a = 1;//do nothing
                    } else {
                        hasinValidChar = true;
                    }
                }
                if (!hasNumber || !hasLetter || hasinValidChar) {
                    model.addAttribute("errorMessage", "密码强度不符合要求，请重新输入。要求：8~32位，需要有数字和字母，允许大小写英文字母、数字、密码常见符号");
                    //FAQ: 开头测试用的账户的密码可能还真不符合要求
                    return "redirect:/user/usermanage";
                }
                // start to create
                Password pw = new Password(password);
                System.out.println("username:" + username + " password:" + pw.getEncryptedPassword() + " role:" + role + " roleid:" + roleid); //debug
                user = new User(username, pw.getEncryptedPassword(), role, (roleid == -1 ? null : roleid));
                int status_code = userMapper.insertUserv2(user);
                if (status_code == 0) {
                    model.addAttribute("errorMessage", "创建用户失败，请重试。");
                } else {
                    model.addAttribute("successMessage", "创建成功");
                }
                return "redirect:/user/usermanage";
            }
        } else {
            model.addAttribute("errorMessage", "用户已存在，请重新输入。");
            return "redirect:/user/usermanage";
        }
    }

    @RequestMapping(path = "/user/reset", method = {RequestMethod.POST})
    public String resetpassword(@RequestParam("id") int id,
                                @RequestParam("password") String password,
                                @RequestParam("re_password") String repassword,
                                Model model) throws NoSuchAlgorithmException {
        //check password
        if (!password.equals(repassword)) {
            model.addAttribute("errorMessage", "两次输入的密码不一致，请重新输入。");
            return "redirect:/user/usermanage";
        }
        //check if user exists
        User user = userMapper.getUserById(id);
        if (user == null) {
            model.addAttribute("errorMessage", "ID为" + id + "的用户不存在，请重新输入。");
            return "redirect:/user/usermanage";
        } else {
            Password pw = new Password(password);
            User user1 = new User(id, pw.getEncryptedPassword());
            int status_code = userMapper.updateUserPassword(user1);
            if (status_code == 0) {
                model.addAttribute("errorMessage", "重置密码失败，请重试。");
            } else {
                model.addAttribute("successMessage", "重置密码成功");
            }
        }
        return "redirect:/user/usermanage";
    }

    @RequestMapping(path = "/user/userdisabled", method = {RequestMethod.POST})
    @ResponseBody
    public Map<String, String> userdisabled(@RequestParam("id") int id,
                                            Model model) {
        Map<String, String> response = new HashMap<>();
        if (userMapper.getUserById(id).getUsername().equals(new UsernameHelper().getCurrentUsername())) {
            response.put("status", "fail");
            response.put("message", "不能禁用自己");
            return response;
        }
        // 执行禁用用户的逻辑
        int status_code = userMapper.disabledUser(id);
        if (status_code == 1) {
            response.put("status", "success");
            response.put("message", "用户禁用成功");
        } else {
            response.put("status", "fail");
            response.put("message", "用户禁用失败");
        }
        return response; // 返回包含成功信息的响应
    }

    @RequestMapping(path = "/user/userenabled", method = {RequestMethod.POST})
    @ResponseBody
    public Map<String, String> userenabled(@RequestParam("id") int id,
                                           Model model) {
        Map<String, String> response = new HashMap<>();
        // 执行启用用户的逻辑
        int status_code = userMapper.enabledUser(id);
        if (status_code == 1) {
            response.put("status", "success");
            response.put("message", "用户启用成功");
        } else {
            response.put("status", "fail");
            response.put("message", "用户启用失败");
        }
        return response; // 返回包含成功信息的响应
    }
}
