package cyou.chenx221.helper;

import cyou.chenx221.pojo.Course;
import cyou.chenx221.pojo.Score;
import cyou.chenx221.pojo.Student;
import cyou.chenx221.pojo.Teacher;
import org.apache.commons.csv.CSVFormat;
import org.apache.commons.csv.CSVPrinter;

import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

public class CsvHelper {
    public String generateScoreCSVFile(List<Score> scores) throws IOException {
        List<String> headers;
        headers = Arrays.asList("成绩ID", "学生ID", "学生姓名", "课程ID", "课程名称", "成绩");
        List<List<String>> data = new ArrayList<>();
        for (Score score : scores) {
            List<String> rowData = Arrays.asList(
                    String.valueOf(score.getScoreID()),
                    String.valueOf(score.getStudent().getId()),
                    score.getStudent().getName(),
                    String.valueOf(score.getCourse().getCourseID()),
                    score.getCourse().getCourseName(),
                    String.valueOf(score.getScore())
            );
            data.add(rowData);
        }
        return getString(headers, data);
    }

    private String getString(List<String> headers, List<List<String>> data) throws IOException {
        String directoryPath = "file";
        File directory = new File(directoryPath);

        if (!directory.exists()) {
            boolean created = directory.mkdir(); //Be careful, path: tomcat's installation path (D:\softs\res\apache-tomcat-9.0.75-windows-x64\apache-tomcat-9.0.75\bin)
            if (created) {
                System.out.println("目录已创建");
            } else {
                System.out.println("无法创建目录");
            }
        }

        File csvFile = new File("file/export-" + LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMdd_HHmmss")) + ".csv");
        try (PrintWriter writer = new PrintWriter(csvFile);
             CSVPrinter csvPrinter = new CSVPrinter(writer, CSVFormat.DEFAULT)) {
            csvPrinter.printRecord(headers);
            for (List<String> row : data) {
                csvPrinter.printRecord(row);
            }
            csvPrinter.flush();
        }
        return csvFile.getName();
    }

    public String generateStudentCSVFile(List<Student> students) throws IOException {
        List<String> headers;
        headers = Arrays.asList("学生ID", "学生姓名", "性别", "出生日期", "联系方式", "班级");
        List<List<String>> data = new ArrayList<>();
        for (Student student : students) {
            List<String> rowData = Arrays.asList(
                    String.valueOf(student.getId()),
                    student.getName(),
                    student.getSex(),
                    student.getBirthday().toString(),
                    String.valueOf(student.getPhone()),
                    String.valueOf(student.getClasses())
            );
            data.add(rowData);
        }
        return getString(headers, data);
    }

    public String generateCourseCSVFile(List<Course> courses) throws IOException {
        List<String> headers;
        headers = Arrays.asList("课程ID", "课程名", "课程描述");
        List<List<String>> data = new ArrayList<>();
        for (Course course : courses) {
            List<String> rowData = Arrays.asList(
                    String.valueOf(course.getCourseID()),
                    course.getCourseName(),
                    course.getDescription()
            );
            data.add(rowData);
        }
        return getString(headers, data);
    }

    public String generateTeacherCSVFile(List<Teacher> teachers) throws IOException {
        List<String> headers;
        headers = Arrays.asList("教师ID", "姓名", "性别","出生日期","授课课程ID","授课课程名","授课班级");
        List<List<String>> data = new ArrayList<>();
        for (Teacher teacher : teachers) {
            List<String> rowData = Arrays.asList(
                    String.valueOf(teacher.getId()),
                    teacher.getName(),
                    teacher.getSex(),
                    teacher.getBirthday().toString(),
                    String.valueOf(teacher.getCourse().getCourseID()),
                    teacher.getCourse().getCourseName(),
                    teacher.getClasses()
            );
            data.add(rowData);
        }
        return getString(headers, data);
    }
}
